/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _IAVLKSS_H
#define _IAVLKSS_H

#include <iglobals.h>
#include <icursor.h>
#include <iiavlkss.h>

template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSet {

  static ElementOps cvElementOps;

  class Node;
  class Operations;
  class Cursor;

  friend class Node;
  friend class Operations;
  friend class Cursor;

  class Node : public IAvlKeySortedSetImpl::Node
  { friend class IGAvlKeySortedSet < Element, Key, ElementOps >;
    friend class Cursor;
    friend class Operations;

    Element ivElement;

                     Node                (Element const& element)
		     : ivElement (element) {};

    void*            operator new        (size_t s)
		     { return cvElementOps.allocate (s); }

    void             operator delete     (void* p)
                     { cvElementOps.deallocate (p); }

  };

  class Operations : public IAvlKeySortedSetImpl::Operations
  { IAvlKeySortedSetImpl::Node*
                newNode                   (void const* element) const;
    IAvlKeySortedSetImpl::Node*
                copyNode                  (void const* node) const;
    void        deleteNode                (void* node) const;
    void*       elementAt                 (void *node) const;
    void const* constElementAt            (void const* node) const;
    void const* keyAt                     (void const* node) const;
    long        compareKeyTo              (void const* node1,
                                           void const* node2) const;
    long        compareKeyToElement       (void const* node,
                                           void const* element) const;
    long        compareKeyToKey           (void const* node,
                                           void const* key) const;
    Boolean     isEqualTo                 (void const* node1,
                                           void const* node2) const;
    Boolean     isEqualToElement          (void const* node,
                                           void const* element) const;
    void        copyFrom                  (void* node,
                                           void const* element) const;

    Boolean     constantFunctionIteration (void *function,
                                           void* env,
                                           void const* node);
    Boolean     functionIteration         (void *function,
                                           void* env,
                                           void* node);
    Boolean     constantIteratorIteration (void* iterator,
                                           void const* node);
    Boolean     iteratorIteration         (void* iterator,
                                           void* node);

    long        functionComparison        (void *compareFunction,
                                           void const* node1,
                                           void const* node2);
  };

  // ivOps must be initialized (and therefore declared) before ivImpl
  // since the constructor of ivImpl uses ivOps.
  Operations ivOps;
  IAvlKeySortedSetImpl ivImpl;

public:

  class Cursor : public ICursor {
  protected:
    IGAvlKeySortedSet < Element, Key, ElementOps > const* ivCollection;
    IAvlKeySortedSetImpl::Node *ivNode;
    friend class IGAvlKeySortedSet < Element, Key, ElementOps >;

  public:
    Cursor (IGAvlKeySortedSet < Element, Key, ElementOps > const& collection)
    : ivCollection (&collection), ivNode (0) {};

    Boolean setToFirst ();
    Boolean setToNext  ();
    Boolean isValid    () const;
    void    invalidate ();

    Element const& element      ()
    { return ivCollection->elementAt (*this); }
    Boolean setToLast     ()
    { return ivCollection->setToLast (*this); }
    Boolean setToPrevious ()
    { return ivCollection->setToPrevious (*this); }
    Boolean operator== (Cursor const& cursor)
    { return ivCollection == cursor.ivCollection &&
             ivNode == cursor.ivNode; }
    Boolean operator!= (Cursor const& cursor)
    { return ! operator== (cursor); }
  };

private:

   void                checkNotEmpty        () const;
   void                checkCursorIsForThis (ICursor const&) const;
   void                checkCursor          (ICursor const&) const;
   void                checkPositionExists  (IPosition) const;

public:

                  IGAvlKeySortedSet             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IGAvlKeySortedSet             (IGAvlKeySortedSet < Element, Key, ElementOps > const&);

   IGAvlKeySortedSet < Element, Key, ElementOps >&    operator =                  (IGAvlKeySortedSet < Element, Key, ElementOps > const&);

                 ~IGAvlKeySortedSet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IGAvlKeySortedSet < Element, Key, ElementOps > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Key const&     key                         (Element const&) const;

   Boolean        containsElementWithKey      (Key const&) const;

   Boolean        containsAllKeysFrom         (IGAvlKeySortedSet < Element, Key, ElementOps > const&) const;

   Boolean        locateElementWithKey        (Key const&, ICursor&)
                                               const;

   Boolean        replaceElementWithKey       (Element const&);

   Boolean        replaceElementWithKey       (Element const&,
                                               ICursor&);

   Boolean        locateOrAddElementWithKey   (Element const&);

   Boolean        locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   Boolean        addOrReplaceElementWithKey  (Element const&);

   Boolean        addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   Boolean        removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

   long           compare                     (IGAvlKeySortedSet < Element, Key, ElementOps > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

};

#include <iavlkss.if>

template < class Element, class Key >
class IAvlKeySortedSet :
  public IGAvlKeySortedSet < Element, Key, IKCOps < Element, Key > > {
public:
  IAvlKeySortedSet (INumber n = 100, IBoundIndicator b = IUnbounded) :
    IGAvlKeySortedSet < Element, Key, IKCOps < Element, Key > > (n, b) {}
};

#ifdef __IBMCPP__
#include <iavlkss.c>
#endif

#endif
