#ifndef _IDRPCTCH_
  #define _IDRPCTCH_
/*--------------------------------------------------------------*/
/* CLASS NAME:    IDropCatcher                                  */
/*                IDropCatcherSet                               */
/*                                                              */
/* DESCRIPTION  : Drop catcher class definition                 */
/*                                                              */
/*                                                              */
/* CHANGE ACTIVITY:                                             */
/* ---------------                                              */
/*   DATE:     INITIAL:       DESCRIPTION:                      */
/* 08/21/92    PHG            Created                           */
/*                                                              */
/*--------------------------------------------------------------*/
/* Copyright (c) IBM Corporation 1991                           */
/*--------------------------------------------------------------*/

// include ICLUI definitions
#ifndef _IBASETYP_
  #include <ibasetyp.hpp>
#endif

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _ILINKLST_
  #include <ilinklst.hpp>
#endif

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif

// include drag&drop-specific headers
#ifndef _IDRGEVT_
  #include <idrgevt.hpp>
#endif
#ifndef _IDRGSTR_
  #include <idrgstr.hpp>
#endif


class IDropCatcher : public IVBase {
/**************************************************************************
* IDropCatcher class                                                      *
*                                                                         *
*   This class is used to process drops.                                  *
*   A collection of drop catchers is registered for each                  *
*   drop handler.                                                         *
*   This collection is iterated for all items to find out if              *
*   they can be handled (during DM_DRAGOVER), and then                    *
*   to actually process the drop (during DM_DROP)                         *
* Hungarian is 'drpcth'                                                   *
**************************************************************************/
friend class IDragTargetHandler;

public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------+
| There is one default way to construct instances of this class:          |
|   1 - Provide types and rendering mechanisms supported by the catcher   |
+------------------------------------------------------------------------*/
  IDropCatcher(const ITypesString& strTypes,
               const IRMFsString& strRMFs,
               Boolean bEndConvRequired=false);
virtual
  ~IDropCatcher();

/*-------------------------------- ACCESSORS -----------------------------+
| These function provide means of getting and setting the accessible      |
| attributes of instances of this class:                                  |
|  types - returns the catcher's supported item types                     |
|  RMFs  - returns the RMFs supported by the catcher                      |
+------------------------------------------------------------------------*/
ITypesString
  types() const;

IRMFsString
  RMFs() const;

/*----------------------------- UTILITY ----------------------------------+
| Utility functions                                                       |
|   postEndConversation - posts a DM_ENDCONVERSATION back to source       |
+------------------------------------------------------------------------*/
Boolean
  postEndConversation(const IWindowHandle& wndhSource,
                      const IDragItem& drgitm,
                      Boolean bSuccess);

/*-------------------------------- OPERATORS -----------------------------+
| operator== - required for GSet1                                         |
+------------------------------------------------------------------------*/
Boolean
  operator==(const IDropCatcher& drpcth);

protected:
/*-------------------------------- CALLBACKS -----------------------------+
| The following functions are intended to be overriden by subclasses to   |
| implement behavior of the drop catcher                                  |
|  isItemDroppable  - check if an item is accepted during DM_DRAGOVER     |
|  isRenderRequired - return rendering RMF, or "" if no rendering required|
|  itemDropped      - called for each item dropped on the target          |
|  itemRendered     - called for items accepted with native RMF           |
|  itemRenderFail   - called for items rendering failed on the source side|
|  itemRendered     - called for items rendered in a given RMF to a name  |
+------------------------------------------------------------------------*/
virtual IEDropFlag
  isItemDroppable(const IDragItem& drgitm,
                  const IDragOperationEvent& drgopevt) const;

virtual IString
  isRenderRequired(const IDragItem& drgitm,
                   const IDragOperationEvent& drgopevt) const;

virtual Boolean
  itemDropped(const IDragTargetHandler* pdrgtgth,
              const IDragItem& drgitm,
              const IDropOnEvent& drponevt);

virtual Boolean
  itemRendered(const IDragItem& drgitm,
               const IDropOnEvent& evt);

virtual void
  itemRenderFail(const IDragItem& drgitm,
                 const IRenderCompleteEvent& rendcompevt);

virtual Boolean
  itemRendered(const IDragItem& drgitm,
               const IRMFsString& strRMFs,
               const IString& strRenderToName,
               unsigned long ulTargetInfo,
               const IRenderCompleteEvent& rendcompevt);

private:
/*------------------------------ DATA MEMBERS ----------------------------+
| Private data members                                                    |
|  strClTypes    - supported Types                                        |
|  strClRMFs     - supported Mechanisms & Formats                         |
|  bClEndConvReq - true if DM_ENDCONVERSATION is required                 |
+------------------------------------------------------------------------*/
  ITypesString strClTypes;
  IRMFsString  strClRMFs;
  Boolean      bClEndConvReq;

}; // IDropCatcher


/*=========================================================================
| Inlines for IDropCatcher                                      PHG 24/10 |
=========================================================================*/
inline IDropCatcher :: IDropCatcher(const ITypesString& strTypes,
                                    const IRMFsString& strRMFs,
                                    Boolean bEndConvRequired)
: strClTypes(strTypes), strClRMFs(strRMFs)
{ bClEndConvReq=bEndConvRequired; }

inline IDropCatcher :: ~IDropCatcher()
{ }

inline ITypesString IDropCatcher :: types() const
{ return strClTypes; }

inline IRMFsString IDropCatcher :: RMFs() const
{ return strClRMFs; }

inline Boolean IDropCatcher :: operator==(const IDropCatcher& drpcth)
{ return &drpcth == this; }

inline Boolean IDropCatcher :: itemRendered(const IDragItem& drgitm,
                                            const IDropOnEvent& drpevt)
{ return(false); }

inline void IDropCatcher :: itemRenderFail(const IDragItem& drgitm,
                                           const IRenderCompleteEvent& rendcevt)
{ }

inline Boolean IDropCatcher ::  itemRendered(const IDragItem& drgitm,
                                             const IRMFsString& strRMFs,
                                             const IString& strRenderToName,
                                             unsigned long ulTargetInfo,
                                             const IRenderCompleteEvent& rendcevt)
{ return(false); }

#endif /* def _IDRPCTCH_ */
