#ifndef _IMENUITEM_
  #define _IMENUITEM_
/**************************************************************/
/* FILE NAME: imnitem.hpp                                     */
/*                                                            */
/* DESCRIPTION:                                               */
/*    Declaration of the class:                               */
/*      IMenuItem - provides a wrapper for individual menu    */
/*      items                                                 */
/*                                                            */
/* COPYRIGHT:                                                 */
/*   Licensed Materials - Property of IBM                     */
/*   (c) Copyright IBM Corporation 1992, 1993                 */
/*   US Government Users Restricted Rights - Use duplication  */
/*   or disclosure restricted by GSA ADP Schedule Contract    */
/*   with IBM Corp.                                           */
/*                                                            */
/*   $Log:   G:/IBMCLASS/IBASEAPP/VCS/IMNITEM.HPV  $                                                    */
// 
//    Rev 1.8   26 Oct 1992 10:06:44   HARPERSP
// Updates for documentation.
//
//   Rev 1.6   20 Oct 1992 16:41:58   HARPERSP
//Make Style and Attribute flag handling consistent with other classes.
//Allow a IMenuItem to switch between Text, Bitmap, Separator, and OwnerDraw.
//Add exeception handling.
/*                                                            */
/**************************************************************/

#ifndef _IHANDLE_
#include <ihandle.hpp>
#endif
#ifndef _ISTRING_
#include <istring.hpp>
#endif
#ifndef _IBITFLAG_
#include <ibitflag.hpp>
#endif

// Forward declarations for other classes
class IMenuItem;   /* mni */
class IResourceId;
class IMenu;
class ISubMenu;


// MENUITEM is a structure declared in PMWIN.H
// It is mapped below to a IMENUITEM for 2 reasons:
//   1) so that this hpp file is not dependant on PM
//   2) so that hwndSubMenu and hItem can be redefined
//      as IWindowHandle and IBitmapHandle. This allows
//      proper reference counting. Otherwise if the PM
//      types were used, assigments would result in a
//      temporary copy.

typedef struct {
  short iPosition;
  unsigned short afStyle;
  unsigned short afAttribute;
  unsigned short id;
  IWindowHandle hwndSubMenu;
  IBitmapHandle hItem;
} IMENUITEM;

class IMenuItem
{

/**********************************************************************
* When it's not possible to completely define a menu via a resource   *
* file then an IMenuItem can be constructed and added to the menu     *
* using the IMenu or ISubMenu member functions.                       *
*                                                                     *
* You can also use IMenu::menuItem() to get a IMenuItem* for an       *
* existing item in a Menu and then manipulate that item using         *
* IMenuItem member functions.                                         *
*                                                                     *
**********************************************************************/

  public:

     INESTEDBITFLAGCLASSDEF0(Style, IMenuItem);
                                  
     static const Style           
      // Mutually exclusive
      breakNoSeparatorLayout,
      breakSeparatorLayout,
      buttonSeparatorLayout,

      // Mutually exclusive
      sysCommandMessageType,
      helpMessageType,

      subMenu,
      noSelect;

      // Setting nothing is the default
      static const Style& defStyle;

     enum Layout { normalLayout, breakNoSeparator, breakSeparator, buttonSeparator };
     void setLayout(Layout layout);

     enum MessageType {command, sysCommand, help};
     void setMessageType(MessageType messageType);

     void setSubMenuItem(Boolean = true);
     void setSelectableItem(Boolean = true);

     INESTEDBITFLAGCLASSDEF0(Attribute, IMenuItem);

     static const Attribute
       // No dependencies here
       noDismiss,
       framed,
       checked,
       disabled,
       hilited;

       // Setting nothing is the default
       static const Attribute& defAttribute;

     void setNoDismiss(Boolean = true);
     void setFramed(Boolean = true);
     void setChecked(Boolean = true);
     void setDisabled(Boolean = true);
     void setHilited(Boolean = true);

     static Style defaultStyle();
     static void  setDefaultStyle(Style newDefault);

     static Attribute defaultAttribute();
     static void      setDefaultAttribute(Attribute newDefault);


/*------------------ CONSTRUCTORS/DESTRUCTORS -------------------------
|  There are 6 ways to construct instances of this class:             |
|                                                                     |
|  1. Create an unintialized item                                     |
|       Such an item should have setText() or setBitMap() applied     |
|       to it before it's displayed in a menu.                        |
|                                                                     |
|  2. Create a text item from a char*                                 |
|                                                                     |
|  3. Create a BitMap item from a IBitMapHandle                       |
|                                                                     |
|  4. Create a Text or BitMap item from an IResourceId                |
|                                                                     |
|  5. Create a special item (Separator or OwnerDraw item)             |
|                                                                     |
|  6. Copy a IMenuItem.                                               |
|                                                                     |
|  Style or attributes can be set when an IMenuItem is constructed.   |
|  These can be OR'ed together and passed in on the constructor.      |
|  All constructors that accept styles or attributes will use         |
|  the default values for the Style or Attributes if these            |
|  parameters are defaulted.                                          |
|  You can set these defaults for all subsequently created            |
|  IMenuItem objects by calling:                                      |
|      setDefaultStyle()                                              |
|      setDefaultAttribute()                                          |
|                                                                     |
|  Once a IMenuItem has been created its Style or Attributes can      |
|  be changes using member functions such as                          |
|     setLayout()                                                     |
|     setMessageType()                                                |
|                                                                     |
|  An IMenuItem object can be created to display one kind of info     |
|  (eg. Text) and subsequently be changed to display a different      |
|  kind (eg. BitMap).                                                 |
|                                                                     |
|  An IMenuItem should not be destroyed until it will no longer       |
|  be displayed on any Menu.                                          |
|                                                                     |
---------------------------------------------------------------------*/

     //
     // Create an uninitialized item
     //
     IMenuItem(const IMenu* const pmnOwner,
               unsigned long ulItemId,
               Style style = defStyle,
               Attribute attribute = defAttribute);

     //
     // Create a Text item from a char*
     //
     IMenuItem(const IMenu* const pmnOwner,
               unsigned long ulItemId,
               const char* const pszItemText,
               Style style = defStyle,
               Attribute attribute = defAttribute);

     //
     // Create a BitMap item from a IBitMapHandle
     //
     IMenuItem(const IMenu* const pmnOwner, 
               unsigned long ulItemId, 
               const IBitmapHandle& hbmpMenuItem,
               Style style = defStyle,
               Attribute attribute = defAttribute);

     enum mniType {Text, Bitmap};

     //
     // Create a Text or BitMap item from a Resource
     //
     IMenuItem(const IMenu* const pmnOwner,
               unsigned long ulItemId,
               const IResourceId& resId, 
               mniType type,
               Style style = defStyle,
               Attribute attribute = defAttribute);

     enum mniTypeSpecial {Separator, OwnerDraw};

     //
     // Create a special Item (Separator or OwnerDraw
     //
     IMenuItem(const IMenu* const pmnOwner,
               mniTypeSpecial type,
               unsigned long ulItemId = 0,
               Style style = defStyle,
               Attribute attribute = defAttribute);

     //
     // Copy constructor
     //
     IMenuItem(const IMenuItem& miCopy);

     virtual ~IMenuItem()     {;}

/*---------------------- ACCESSORS ------------------------------------
| id - returns the ID associated with the IMenuItem                   |
|                                                                     |
| setText - sets the text to be displayed                             |
| setBitMap - sets the BitMap to be displayed                         |
| setSeparator - makes the Item display a Menu Separator              |
| setOwnerDraw - the user must now provide the routine to draw item   |
|                                                                     |
| text() - returns an IString representing the text to be displayed   |
| bitMap - returns the IBitMapHandle of the item                      |
---------------------------------------------------------------------*/

     unsigned long      id() const {return ClIMENUITEM.id;}

     void               setText(const char* const pszNewText);
     void               setText(const IResourceId& textResId);

     void               setBitMap(const IBitmapHandle& hbmpMenuItem);
     void               setBitMap(const IResourceId& bitmapResId);
     void               setBitMap(unsigned long      bitmapResId);

     void               setSeparator();
     void               setOwnerDraw();

     IString            text();
     IBitmapHandle      bitMap();

    protected:

     static Style classDefaultStyle;
     static Attribute classDefaultAttribute;

     unsigned long      style();
     void               setStyle(unsigned long newStyle);

     unsigned long      attribute();
     void               setAttribute(unsigned long newAttribute);


/*----------------------- PRIVATE -----------------------------------*/
    private:
 
     const   IMenu*     pmnClOwner;
     IMENUITEM          ClIMENUITEM;

     // Need to store as an IString so that reference counting works
     IString            strClItemText;

     friend  class      IMenu;
     friend  class      ISubMenu;

     // not allowing assignments
     IMenuItem& operator=(const IMenuItem&);

     //
     // Special constructor used by IMenu
     // to construct a IMenuItem given an ItemId
     //
     IMenuItem(const IMenu* const pmnOwner,
               IMENUITEM* pIMENUITEM,
               char* pszItemText = 0);

     void  update(const IMENUITEM& newMENUTITEM);
     void  setItem();
};


#endif  /* _IMENUITEM_ */
